@@ -1,11 +1,11 @@
 /* compress.c - bzip2 compress filter
- * Copyright (C) 2003, 2004 Free Software Foundation, Inc.
+ * Copyright (C) 2003 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
  * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 3 of the License, or
+ * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * GnuPG is distributed in the hope that it will be useful,
@@ -14,12 +14,12 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
 #include <config.h>
 #include <string.h>
-#include <stdio.h> /* Early versions of bzlib (1.0) require stdio.h */
 #include <bzlib.h>
 
 #include "util.h"
@@ -41,9 +41,9 @@ init_compress( compress_filter_context_t *zfx, bz_stream *bzs )
   int rc;
   int level;
 
-  if( opt.bz2_compress_level >= 1 && opt.bz2_compress_level <= 9 )
-    level = opt.bz2_compress_level;
-  else if( opt.bz2_compress_level == -1 )
+  if( opt.compress >= 1 && opt.compress <= 9 )
+    level = opt.compress;
+  else if( opt.compress == -1 )
     level = 6; /* no particular reason, but it seems reasonable */
   else
     {
@@ -55,7 +55,7 @@ init_compress( compress_filter_context_t *zfx, bz_stream *bzs )
     log_fatal("bz2lib problem: %d\n",rc);
 
   zfx->outbufsize = 8192;
-  zfx->outbuf = xmalloc( zfx->outbufsize );
+  zfx->outbuf = m_alloc( zfx->outbufsize );
 }
 
 static int
@@ -100,11 +100,11 @@ init_uncompress( compress_filter_context_t *zfx, bz_stream *bzs )
 {
   int rc;
 
-  if((rc=BZ2_bzDecompressInit(bzs,0,opt.bz2_decompress_lowmem))!=BZ_OK)
+  if((rc=BZ2_bzDecompressInit(bzs,0,0))!=BZ_OK)
     log_fatal("bz2lib problem: %d\n",rc);
 
   zfx->inbufsize = 2048;
-  zfx->inbuf = xmalloc( zfx->inbufsize );
+  zfx->inbuf = m_alloc( zfx->inbufsize );
   bzs->avail_in = 0;
 }
 
@@ -117,7 +117,6 @@ do_uncompress( compress_filter_context_t *zfx, bz_stream *bzs,
   size_t n;
   int nread, count;
   int refill = !bzs->avail_in;
-  int eofseen = 0;
 
   if( DBG_FILTER )
     log_debug("begin bzDecompress: avail_in=%u, avail_out=%u, inbuf=%u\n",
@@ -132,16 +131,12 @@ do_uncompress( compress_filter_context_t *zfx, bz_stream *bzs,
 	    bzs->next_in = zfx->inbuf;
 	  count = zfx->inbufsize - n;
 	  nread = iobuf_read( a, zfx->inbuf + n, count );
-	  if( nread == -1 )
-            {
-              eofseen = 1;
-              nread = 0;
-            }
+	  if( nread == -1 ) nread = 0;
 	  n += nread;
 	  bzs->avail_in = n;
 	}
-      if (!eofseen)
-        refill = 1;
+
+      refill = 1;
 
       if( DBG_FILTER )
 	log_debug("enter bzDecompress: avail_in=%u, avail_out=%u\n",
@@ -155,13 +150,6 @@ do_uncompress( compress_filter_context_t *zfx, bz_stream *bzs,
 	rc = -1; /* eof */
       else if( zrc != BZ_OK && zrc != BZ_PARAM_ERROR )
 	log_fatal("bz2lib inflate problem: rc=%d\n", zrc );
-      else if (zrc == BZ_OK && eofseen 
-               && !bzs->avail_in && bzs->avail_out > 0)
-        {
-          log_error ("unexpected EOF in bz2lib\n");
-          rc = G10ERR_READ_FILE;
-          break;
-        }
     }
   while( bzs->avail_out && zrc != BZ_STREAM_END && zrc != BZ_PARAM_ERROR );
 
@@ -188,7 +176,7 @@ compress_filter_bz2( void *opaque, int control,
     {
       if( !zfx->status )
 	{
-	  bzs = zfx->opaque = xmalloc_clear( sizeof *bzs );
+	  bzs = zfx->opaque = m_alloc_clear( sizeof *bzs );
 	  init_uncompress( zfx, bzs );
 	  zfx->status = 1;
 	}
@@ -215,7 +203,7 @@ compress_filter_bz2( void *opaque, int control,
 	  pkt.pkt.compressed = &cd;
 	  if( build_packet( a, &pkt ))
 	    log_bug("build_packet(PKT_COMPRESSED) failed\n");
-	  bzs = zfx->opaque = xmalloc_clear( sizeof *bzs );
+	  bzs = zfx->opaque = m_alloc_clear( sizeof *bzs );
 	  init_compress( zfx, bzs );
 	  zfx->status = 2;
 	}
@@ -229,9 +217,9 @@ compress_filter_bz2( void *opaque, int control,
       if( zfx->status == 1 )
 	{
 	  BZ2_bzDecompressEnd(bzs);
-	  xfree(bzs);
+	  m_free(bzs);
 	  zfx->opaque = NULL;
-	  xfree(zfx->outbuf); zfx->outbuf = NULL;
+	  m_free(zfx->outbuf); zfx->outbuf = NULL;
 	}
       else if( zfx->status == 2 )
 	{
@@ -239,14 +227,14 @@ compress_filter_bz2( void *opaque, int control,
 	  bzs->avail_in = 0;
 	  do_compress( zfx, bzs, BZ_FINISH, a );
 	  BZ2_bzCompressEnd(bzs);
-	  xfree(bzs);
+	  m_free(bzs);
 	  zfx->opaque = NULL;
-	  xfree(zfx->outbuf); zfx->outbuf = NULL;
+	  m_free(zfx->outbuf); zfx->outbuf = NULL;
 	}
       if (zfx->release)
 	zfx->release (zfx);
     }
   else if( control == IOBUFCTRL_DESC )
-    mem2str (buf, "compress_filter", *ret_len);
+    *(char**)buf = "compress_filter";
   return rc;
 }
