@@ -1,5 +1,5 @@
 /* decrypt.c - verify signed data
- * Copyright (C) 1998, 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
+ * Copyright (C) 1998,1999,2000,2001,2002,2003 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -51,6 +51,7 @@ decrypt_message( const char *filename )
 {
     IOBUF fp;
     armor_filter_context_t afx;
+    progress_filter_context_t pfx;
     int rc;
     int no_out=0;
 
@@ -61,6 +62,8 @@ decrypt_message( const char *filename )
 	return G10ERR_OPEN_FILE;
     }
 
+    handle_progress (&pfx, fp, filename);
+
     if( !opt.no_armor ) {
 	if( use_armor_filter( fp ) ) {
 	    memset( &afx, 0, sizeof afx);
@@ -84,6 +87,7 @@ decrypt_messages(int nfiles, char **files)
 {
   IOBUF fp;
   armor_filter_context_t afx;  
+  progress_filter_context_t pfx;
   char *p, *output = NULL;
   int rc = 0;
   
@@ -99,13 +103,16 @@ decrypt_messages(int nfiles, char **files)
       print_file_status(STATUS_FILE_START, *files, 3);      
       output = make_outfile_name(*files);
       if (!output)
-        continue;
+        goto next_file;
       fp = iobuf_open(*files);
       if (!fp)
         {
           log_error(_("can't open `%s'\n"), print_fname_stdin(*files));
-          continue;
+          goto next_file;
         }
+
+      handle_progress (&pfx, fp, *files);
+
       if (!opt.no_armor)
         {
           if (use_armor_filter(fp))
@@ -122,17 +129,12 @@ decrypt_messages(int nfiles, char **files)
       p = get_last_passphrase();
       set_next_passphrase(p);
       m_free (p);
-      files++;
-      m_free(output);
+     
+    next_file:
+      /* Note that we emit file_done even after an error. */
       write_status( STATUS_FILE_DONE );
+      m_free(output);
+      files++;
     }
   set_next_passphrase(NULL);  
 }
-
-
-
-
-
-
-
-
