@@ -38,7 +38,7 @@
 #include "i18n.h"
 #include "status.h"
 
-static int encode_simple( const char *filename, int mode, int compat );
+static int encode_simple( const char *filename, int mode, int use_seskey );
 static int write_pubkey_enc_from_list( PK_LIST pk_list, DEK *dek, IOBUF out );
 
 
@@ -50,17 +50,7 @@ static int write_pubkey_enc_from_list( PK_LIST pk_list, DEK *dek, IOBUF out );
 int
 encode_symmetric( const char *filename )
 {
-    int compat = 1;
-
-#if 0    
-    /* We don't want to use it because older gnupg version can't
-       handle it and we can presume that a lot of scripts are running
-       with the expert mode set.  Some time in the future we might
-       want to allow for it. */
-    if ( opt.expert )
-        compat = 0; /* PGP knows how to handle this mode. */
-#endif
-    return encode_simple( filename, 1, compat );
+    return encode_simple( filename, 1, 0 );
 }
 
 /****************
@@ -70,7 +60,7 @@ encode_symmetric( const char *filename )
 int
 encode_store( const char *filename )
 {
-    return encode_simple( filename, 0, 1 );
+    return encode_simple( filename, 0, 0 );
 }
 
 static void
@@ -80,7 +70,7 @@ encode_sesskey( DEK *dek, DEK **ret_dek, byte *enckey )
     DEK *c;
     byte buf[33];
 
-    assert ( dek->keylen < 32 );
+    assert ( dek->keylen <= 32 );
     
     c = m_alloc_clear( sizeof *c );
     c->keylen = dek->keylen;
@@ -106,6 +96,10 @@ encode_sesskey( DEK *dek, DEK **ret_dek, byte *enckey )
 static int
 use_mdc(PK_LIST pk_list,int algo)
 {
+  /* RFC-1991 and 2440 don't have MDC */
+  if(RFC1991 || RFC2440)
+    return 0;
+
   /* --force-mdc overrides --disable-mdc */
   if(opt.force_mdc)
     return 1;
@@ -148,8 +142,12 @@ use_mdc(PK_LIST pk_list,int algo)
   return 0; /* No MDC */
 }
 
+/* We don't want to use use_seskey yet because older gnupg versions
+   can't handle it, and there isn't really any point unless we're
+   making a message that can be decrypted by a public key or
+   passphrase. */
 static int
-encode_simple( const char *filename, int mode, int compat )
+encode_simple( const char *filename, int mode, int use_seskey )
 {
     IOBUF inp, out;
     PACKET pkt;
@@ -165,7 +163,7 @@ encode_simple( const char *filename, int mode, int compat )
     compress_filter_context_t zfx;
     text_filter_context_t tfx;
     progress_filter_context_t pfx;
-    int do_compress = opt.compress && !opt.rfc1991;
+    int do_compress = opt.compress && !RFC1991;
 
     memset( &cfx, 0, sizeof cfx);
     memset( &afx, 0, sizeof afx);
@@ -188,13 +186,13 @@ encode_simple( const char *filename, int mode, int compat )
     /* Due the the fact that we use don't use an IV to encrypt the
        session key we can't use the new mode with RFC1991 because
        it has no S2K salt. RFC1991 always uses simple S2K. */
-    if ( opt.rfc1991 && !compat )
-        compat = 1;
+    if ( RFC1991 && use_seskey )
+        use_seskey = 0;
     
     cfx.dek = NULL;
     if( mode ) {
 	s2k = m_alloc_clear( sizeof *s2k );
-	s2k->mode = opt.rfc1991? 0:opt.s2k_mode;
+	s2k->mode = RFC1991? 0:opt.s2k_mode;
 	s2k->hash_algo = opt.def_digest_algo ? opt.def_digest_algo
 					     : opt.s2k_digest_algo;
 	cfx.dek = passphrase_to_dek( NULL, 0,
@@ -209,16 +207,14 @@ encode_simple( const char *filename, int mode, int compat )
 	    log_error(_("error creating passphrase: %s\n"), g10_errstr(rc) );
 	    return rc;
 	}
-        if (!compat && s2k->mode != 1 && s2k->mode != 3) {
-            compat = 1;
+        if (use_seskey && s2k->mode != 1 && s2k->mode != 3) {
+            use_seskey = 0;
             log_info (_("can't use a symmetric ESK packet "
                         "due to the S2K mode\n"));
         }
 
-        if ( !compat ) {            
-            seskeylen = cipher_get_keylen( opt.def_cipher_algo ?
-                                           opt.def_cipher_algo:
-                                           opt.s2k_cipher_algo ) / 8;
+        if ( use_seskey ) {            
+            seskeylen = cipher_get_keylen( opt.s2k_cipher_algo ) / 8;
             encode_sesskey( cfx.dek, &dek, enckey );
             m_free( cfx.dek ); cfx.dek = dek;
         }
@@ -243,20 +239,20 @@ encode_simple( const char *filename, int mode, int compat )
 
     if( opt.armor )
 	iobuf_push_filter( out, armor_filter, &afx );
-  #ifdef ENABLE_COMMENT_PACKETS
+#ifdef ENABLE_COMMENT_PACKETS
     else {
 	write_comment( out, "#created by GNUPG v" VERSION " ("
 					    PRINTABLE_OS_NAME ")");
 	if( opt.comment_string )
 	    write_comment( out, opt.comment_string );
     }
-  #endif
-    if( s2k && !opt.rfc1991 ) {
+#endif
+    if( s2k && !RFC1991 ) {
 	PKT_symkey_enc *enc = m_alloc_clear( sizeof *enc + seskeylen + 1 );
 	enc->version = 4;
 	enc->cipher_algo = cfx.dek->algo;
 	enc->s2k = *s2k;
-        if ( !compat && seskeylen ) {
+        if ( use_seskey && seskeylen ) {
             enc->seskeylen = seskeylen + 1; /* algo id */
             memcpy( enc->seskey, enckey, seskeylen + 1 );
         }
@@ -294,7 +290,8 @@ encode_simple( const char *filename, int mode, int compat )
        either partial length or fixed length with the new style
        messages. */
 
-    if( filename && !opt.textmode ) {
+    if (filename && *filename && !(*filename == '-' && !filename[1])
+        && !opt.textmode ) {
         off_t tmpsize;
 
 	if ( !(tmpsize = iobuf_get_filelength(inp)) )
@@ -315,7 +312,7 @@ encode_simple( const char *filename, int mode, int compat )
 	pt->timestamp = make_timestamp();
 	pt->mode = opt.textmode? 't' : 'b';
 	pt->len = filesize;
-	pt->new_ctb = !pt->len && !opt.rfc1991;
+	pt->new_ctb = !pt->len && !RFC1991;
 	pt->buf = inp;
 	pkt.pkttype = PKT_PLAINTEXT;
 	pkt.pkt.plaintext = pt;
@@ -336,10 +333,7 @@ encode_simple( const char *filename, int mode, int compat )
       {
         if (cfx.dek && cfx.dek->use_mdc)
           zfx.new_ctb = 1;
-	zfx.algo=opt.def_compress_algo;
-	if(zfx.algo==-1)
-	  zfx.algo=DEFAULT_COMPRESS_ALGO;
-	iobuf_push_filter( out, compress_filter, &zfx );
+	push_compress_filter(out,&zfx,opt.def_compress_algo);
       }
 
     /* do the work */
@@ -396,7 +390,7 @@ encode_crypt( const char *filename, STRLIST remusr )
     text_filter_context_t tfx;
     progress_filter_context_t pfx;
     PK_LIST pk_list,work_list;
-    int do_compress = opt.compress && !opt.rfc1991;
+    int do_compress = opt.compress && !RFC1991;
 
 
     memset( &cfx, 0, sizeof cfx);
@@ -408,15 +402,14 @@ encode_crypt( const char *filename, STRLIST remusr )
     if( (rc=build_pk_list( remusr, &pk_list, PUBKEY_USAGE_ENC)) )
 	return rc;
 
-    if(opt.pgp2) {
+    if(PGP2) {
       for(work_list=pk_list; work_list; work_list=work_list->next)
 	if(!(is_RSA(work_list->pk->pubkey_algo) &&
 	     nbits_from_pk(work_list->pk)<=2048))
 	  {
 	    log_info(_("you can only encrypt to RSA keys of 2048 bits or "
 		       "less in --pgp2 mode\n"));
-	    log_info(_("this message may not be usable by %s\n"),"PGP 2.x");
-	    opt.pgp2=0;
+	    compliance_failure();
 	    break;
 	  }
     }
@@ -442,14 +435,14 @@ encode_crypt( const char *filename, STRLIST remusr )
 
     if( opt.armor )
 	iobuf_push_filter( out, armor_filter, &afx );
-  #ifdef ENABLE_COMMENT_PACKETS
+#ifdef ENABLE_COMMENT_PACKETS
     else {
 	write_comment( out, "#created by GNUPG v" VERSION " ("
 					    PRINTABLE_OS_NAME ")");
 	if( opt.comment_string )
 	    write_comment( out, opt.comment_string );
     }
-  #endif
+#endif
     /* create a session key */
     cfx.dek = m_alloc_secure_clear (sizeof *cfx.dek);
     if( !opt.def_cipher_algo ) { /* try to get it from the prefs */
@@ -464,11 +457,10 @@ encode_crypt( const char *filename, STRLIST remusr )
 	if( cfx.dek->algo == -1 ) {
 	    cfx.dek->algo = CIPHER_ALGO_3DES;
 
-	    if( opt.pgp2 ) {
+	    if( PGP2 ) {
 	      log_info(_("unable to use the IDEA cipher for all of the keys "
 			 "you are encrypting to.\n"));
-	      log_info(_("this message may not be usable by %s\n"),"PGP 2.x");
-	      opt.pgp2=0;
+	      compliance_failure();
 	    }
 	}
     }
@@ -527,7 +519,8 @@ encode_crypt( const char *filename, STRLIST remusr )
 	}
     }
 
-    if( filename && !opt.textmode ) {
+    if (filename && *filename && !(*filename == '-' && !filename[1])
+        && !opt.textmode ) {
         off_t tmpsize;
 
 	if ( !(tmpsize = iobuf_get_filelength(inp)) )
@@ -548,7 +541,7 @@ encode_crypt( const char *filename, STRLIST remusr )
 	pt->timestamp = make_timestamp();
 	pt->mode = opt.textmode ? 't' : 'b';
 	pt->len = filesize;
-	pt->new_ctb = !pt->len && !opt.rfc1991;
+	pt->new_ctb = !pt->len && !RFC1991;
 	pt->buf = inp;
 	pkt.pkttype = PKT_PLAINTEXT;
 	pkt.pkt.plaintext = pt;
@@ -582,8 +575,7 @@ encode_crypt( const char *filename, STRLIST remusr )
 	  {
             if (cfx.dek && cfx.dek->use_mdc)
               zfx.new_ctb = 1;
-	    zfx.algo = compr_algo;
-	    iobuf_push_filter( out, compress_filter, &zfx );
+	    push_compress_filter(out,&zfx,compr_algo);
 	  }
     }
 
@@ -716,16 +708,11 @@ write_pubkey_enc_from_list( PK_LIST pk_list, DEK *dek, IOBUF out )
 	keyid_from_pk( pk, enc->keyid );
 	enc->throw_keyid = opt.throw_keyid;
 
-	if(opt.throw_keyid && (opt.pgp2 || opt.pgp6 || opt.pgp7 || opt.pgp8))
+	if(opt.throw_keyid && (PGP2 || PGP6 || PGP7 || PGP8))
 	  {
 	    log_info(_("you may not use %s while in %s mode\n"),
-		     "--throw-keyid",
-		     opt.pgp2?"--pgp2":opt.pgp6?"--pgp6":opt.pgp7?"--pgp7":"--pgp8");
-
-	    log_info(_("this message may not be usable by %s\n"),
-		     opt.pgp2?"PGP 2.x":opt.pgp6?"PGP 6.x":opt.pgp7?"PGP 7.x":"PGP 8.x");
-
-	    opt.pgp2=opt.pgp6=opt.pgp7=opt.pgp8=0;
+		     "--throw-keyid",compliance_option_string());
+	    compliance_failure();
 	  }
 
 	/* Okay, what's going on: We have the session key somewhere in
