@@ -68,7 +68,6 @@ free_seckey_enc( PKT_signature *sig )
 }
 
 
-
 void
 release_public_key_parts( PKT_public_key *pk )
 {
@@ -84,10 +83,6 @@ release_public_key_parts( PKT_public_key *pk )
         m_free (pk->prefs);
         pk->prefs = NULL;
     }
-    if( pk->namehash ) {
-	m_free(pk->namehash);
-	pk->namehash = NULL;
-    }
     if (pk->user_id) {
         free_user_id (pk->user_id);
         pk->user_id = NULL;
