@@ -1,5 +1,6 @@
 /* g10.c - The GnuPG utility (main for gpg)
- * Copyright (C) 1998,1999,2000,2001,2002,2003 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002,
+ *               2003 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -51,13 +52,15 @@
 #include "keyserver-internal.h"
 #include "exec.h"
 
-enum cmd_and_opt_values { aNull = 0,
+enum cmd_and_opt_values
+  {
+    aNull = 0,
     oArmor	  = 'a',
     aDetachedSign = 'b',
     aSym	  = 'c',
     aDecrypt	  = 'd',
     aEncr	  = 'e',
-    aEncrFiles,
+    aOldEncrFiles = 'f', /* to be removed */
     oInteractive  = 'i',
     oKOption	  = 'k',
     oDryRun	  = 'n',
@@ -75,6 +78,7 @@ enum cmd_and_opt_values { aNull = 0,
     oCertNotation,
     oShowNotation,
     oNoShowNotation,
+    aEncrFiles,
     aDecryptFiles,                          
     aClearsign,
     aStore,
@@ -156,6 +160,7 @@ enum cmd_and_opt_values { aNull = 0,
     oAttributeFile, /* only used with risc os */
     oSKComments,
     oNoSKComments,
+    oNoComments,
     oEmitVersion,
     oNoEmitVersion,
     oCompletesNeeded,
@@ -292,12 +297,15 @@ enum cmd_and_opt_values { aNull = 0,
     oLCctype,
     oLCmessages,
     oGroup,
+    oNoGroups,
     oStrict,
     oNoStrict,
     oMangleDosFilenames,
     oNoMangleDosFilenames,
-    oEnableProgressFilter,                         
-aTest };
+    oEnableProgressFilter,
+    oMultifile,
+    aTest
+  };
 
 
 static ARGPARSE_OPTS opts[] = {
@@ -305,9 +313,10 @@ static ARGPARSE_OPTS opts[] = {
     { 300, NULL, 0, N_("@Commands:\n ") },
 
     { aSign, "sign",      256, N_("|[file]|make a signature")},
-    { aClearsign, "clearsign", 256, N_("|[file]|make a clear text signature") },
+    { aClearsign, "clearsign", 256, N_("|[file]|make a clear text signature")},
     { aDetachedSign, "detach-sign", 256, N_("make a detached signature")},
     { aEncr, "encrypt",   256, N_("encrypt data")},
+    { aOldEncrFiles, NULL, 256, "@"},
     { aEncrFiles, "encrypt-files", 256, N_("|[files]|encrypt files")},
     { aSym, "symmetric", 256, N_("encryption only with symmetric cipher")},
     { aStore, "store",     256, N_("store only")},
@@ -378,7 +387,7 @@ static ARGPARSE_OPTS opts[] = {
     { oEncryptTo, "encrypt-to", 2, "@" },
     { oNoEncryptTo, "no-encrypt-to", 0, "@" },
     { oUser, "local-user",2, N_("use this user-id to sign or decrypt")},
-    { oCompress, NULL,	      1, N_("|N|set compress level N (0 disables)") },
+    { oCompress, "compress-level", 1, N_("|N|set compress level N (0 disables)") },
     { oTextmodeShort, NULL,   0, "@"},
     { oTextmode, "textmode",  0, N_("use canonical text mode")},
     { oNoTextmode, "no-textmode",  0, "@"},
@@ -429,7 +438,7 @@ static ARGPARSE_OPTS opts[] = {
 #ifdef __riscos__
     { oAttributeFile, "attribute-file" ,2, "@" },
 #endif /* __riscos__ */
-    { oNoSKComments, "no-comment", 0,   "@"},
+    { oNoComments, "no-comment", 0,   "@"},
     { oNoSKComments, "no-sk-comments", 0,   "@"},
     { oSKComments, "sk-comments", 0,   "@"},
     { oCompletesNeeded, "completes-needed", 1, "@"},
@@ -595,11 +604,13 @@ static ARGPARSE_OPTS opts[] = {
     { oLCctype,    "lc-ctype",    2, "@" },
     { oLCmessages, "lc-messages", 2, "@" },
     { oGroup,      "group",       2, "@" },
+    { oNoGroups,   "no-groups",    0, "@" },
     { oStrict,     "strict",      0, "@" },
     { oNoStrict,   "no-strict",   0, "@" },
     { oMangleDosFilenames, "mangle-dos-filenames", 0, "@" },
     { oNoMangleDosFilenames, "no-mangle-dos-filenames", 0, "@" },
     { oEnableProgressFilter, "enable-progress-filter", 0, "@" },
+    { oMultifile, "multifile", 0, "@" },
 {0} };
 
 
@@ -634,6 +645,19 @@ strusage( int level )
       case 19: p =
 	    _("Please report bugs to <gnupg-bugs@gnu.org>.\n");
 	break;
+
+#ifdef IS_DEVELOPMENT_VERSION
+      case 20:
+	p="NOTE: THIS IS A DEVELOPMENT VERSION!";
+	break;
+      case 21:
+	p="It is only intended for test purposes and should NOT be";
+	break;
+      case 22:
+	p="used in a production environment or with production keys!";
+	break;
+#endif
+
       case 1:
       case 40:	p =
 	    _("Usage: gpg [options] [files] (-h for help)");
@@ -1071,6 +1095,7 @@ main( int argc, char **argv )
     char *pers_digest_list = NULL;
     char *pers_compress_list = NULL;
     int eyes_only=0;
+    int multifile=0;
     int pwfd = -1;
     int with_fpr = 0; /* make an option out of --fingerprint */
     int any_explicit_recipient = 0;
@@ -1246,8 +1271,10 @@ main( int argc, char **argv )
     }
 
     while( optfile_parse( configfp, configname, &configlineno,
-						&pargs, opts) ) {
-	switch( pargs.r_opt ) {
+						&pargs, opts) )
+      {
+	switch( pargs.r_opt )
+	  {
 	  case aCheckKeys: set_cmd( &cmd, aCheckKeys); break;
 	  case aListPackets: set_cmd( &cmd, aListPackets); break;
 	  case aImport: set_cmd( &cmd, aImport); break;
@@ -1257,7 +1284,10 @@ main( int argc, char **argv )
 	  case aSearchKeys: set_cmd( &cmd, aSearchKeys); break;
 	  case aRefreshKeys: set_cmd( &cmd, aRefreshKeys); break;
 	  case aExport: set_cmd( &cmd, aExport); break;
-	  case aExportAll: set_cmd( &cmd, aExportAll); break;
+	  case aExportAll:
+	    opt.export_options|=EXPORT_INCLUDE_NON_RFC;
+	    set_cmd(&cmd,aExport);
+	    break;
 	  case aListKeys: set_cmd( &cmd, aListKeys); break;
 	  case aListSigs: set_cmd( &cmd, aListSigs); break;
 	  case aExportSecret: set_cmd( &cmd, aExportSecret); break;
@@ -1273,11 +1303,18 @@ main( int argc, char **argv )
 	  case aDetachedSign: detached_sig = 1; set_cmd( &cmd, aSign ); break;
 	  case aSym: set_cmd( &cmd, aSym); break;
 
+	  case aDecryptFiles: multifile=1; /* fall through */
 	  case aDecrypt: set_cmd( &cmd, aDecrypt); break;
-          case aDecryptFiles: set_cmd( &cmd, aDecryptFiles); break;
 
+	  case aOldEncrFiles:
+	    deprecated_warning(configname,configlineno,
+			       "-f","--encrypt-files",""); /* fall through */
+	  case aEncrFiles: multifile=1; /* fall through again */
 	  case aEncr: set_cmd( &cmd, aEncr); break;
-	  case aEncrFiles: set_cmd( &cmd, aEncrFiles ); break;
+
+	  case aVerifyFiles: multifile=1; /* fall through */
+	  case aVerify: set_cmd( &cmd, aVerify); break;
+
 	  case aSign: set_cmd( &cmd, aSign );  break;
 	  case aKeygen: set_cmd( &cmd, aKeygen); greeting=1; break;
 	  case aSignKey: set_cmd( &cmd, aSignKey); break;
@@ -1289,8 +1326,6 @@ main( int argc, char **argv )
 	  case aClearsign: set_cmd( &cmd, aClearsign); break;
 	  case aGenRevoke: set_cmd( &cmd, aGenRevoke); break;
 	  case aDesigRevoke: set_cmd( &cmd, aDesigRevoke); break;
-	  case aVerify: set_cmd( &cmd, aVerify); break;
-	  case aVerifyFiles: set_cmd( &cmd, aVerifyFiles); break;
 	  case aPrimegen: set_cmd( &cmd, aPrimegen); break;
 	  case aGenRandom: set_cmd( &cmd, aGenRandom); break;
 	  case aPrintMD: set_cmd( &cmd, aPrintMD); break;
@@ -1384,6 +1419,10 @@ main( int argc, char **argv )
 			   opt.verbose = 0; opt.list_sigs=0; break;
 	  case oQuickRandom: quick_random_gen(1); break;
 	  case oSKComments: opt.sk_comments=1; break;
+	  case oNoComments:
+	    deprecated_warning(configname,configlineno,
+			       "--no-comment","--no-sk-comments","");
+	    /* fall through */
 	  case oNoSKComments: opt.sk_comments=0; break;
 	  case oEmitVersion: opt.no_version=0; break;
 	  case oNoEmitVersion: opt.no_version=1; break;
@@ -1440,7 +1479,6 @@ main( int argc, char **argv )
 	  case oRFC1991:
 	    opt.compliance = CO_RFC1991;
 	    opt.force_v4_certs = 0;
-	    opt.disable_mdc = 1;
 	    opt.escape_from = 1;
 	    break;
 	  case oRFC2440:
@@ -1448,7 +1486,6 @@ main( int argc, char **argv )
 	    /* TODO: When 2440bis becomes a RFC, these may need
                changing. */
 	    opt.compliance = CO_RFC2440;
-	    opt.disable_mdc = 1;
 	    opt.allow_non_selfsigned_uid = 1;
 	    opt.allow_freeform_uid = 1;
 	    opt.pgp2_workarounds = 0;
@@ -1692,17 +1729,25 @@ main( int argc, char **argv )
           case oLCctype: opt.lc_ctype = pargs.r.ret_str; break;
           case oLCmessages: opt.lc_messages = pargs.r.ret_str; break;
 	  case oGroup: add_group(pargs.r.ret_str); break;
+	  case oNoGroups:
+	    while(opt.grouplist)
+	      {
+		struct groupitem *iter=opt.grouplist;
+		free_strlist(iter->values);
+		opt.grouplist=opt.grouplist->next;
+		m_free(iter);
+	      }
+	    break;
 	  case oStrict: /* noop */ break;
 	  case oNoStrict: /* noop */ break;
-
           case oMangleDosFilenames: opt.mangle_dos_filenames = 1; break;
-          case oNoMangleDosFilenames: opt.mangle_dos_filenames = 0; break;
-
-        case oEnableProgressFilter: opt.enable_progress_filter = 1; break;
+	  case oNoMangleDosFilenames: opt.mangle_dos_filenames = 0; break;
+ 	  case oEnableProgressFilter: opt.enable_progress_filter = 1; break;
+	  case oMultifile: multifile=1; break;
 
 	  default : pargs.err = configfp? 1:2; break;
-	}
-    }
+	  }
+      }
 
     if( configfp ) {
 	fclose( configfp );
@@ -1722,11 +1767,17 @@ main( int argc, char **argv )
 	fprintf(stderr, "%s\n", strusage(15) );
     }
 #ifdef IS_DEVELOPMENT_VERSION
-    if( !opt.batch ) {
-	log_info("NOTE: THIS IS A DEVELOPMENT VERSION!\n");
-	log_info("It is only intended for test purposes and should NOT be\n");
-	log_info("used in a production environment or with production keys!\n");
-    }
+    if( !opt.batch )
+      {
+	const char *s;
+
+	if((s=strusage(20)))
+	  log_info("%s\n",s);
+	if((s=strusage(21)))
+	  log_info("%s\n",s);
+	if((s=strusage(22)))
+	  log_info("%s\n",s);
+      }
 #endif
 
     if (opt.verbose > 2)
@@ -1822,8 +1873,6 @@ main( int argc, char **argv )
 	  compliance_failure();
 	else
 	  {
-	    opt.force_mdc = 0;
-	    opt.disable_mdc = 1;
 	    opt.force_v4_certs = 0;
 	    opt.sk_comments = 0;
 	    opt.escape_from = 1;
@@ -1842,8 +1891,6 @@ main( int argc, char **argv )
 	opt.escape_from=1;
 	opt.force_v3_sigs=1;
 	opt.ask_sig_expire=0;
-	opt.force_mdc=0;
-	opt.disable_mdc=1;
       }
     else if(PGP7)
       {
@@ -1892,6 +1939,11 @@ main( int argc, char **argv )
 	if( check_digest_algo(opt.s2k_digest_algo) )
 	    log_error(_("selected digest algorithm is invalid\n"));
     }
+#ifdef HAVE_BZIP2
+    if(opt.def_compress_algo==3)
+      log_info(_("compress algorithm `%s' is read-only in this release\n"),
+	       "BZIP2");
+#endif
     if( opt.def_compress_algo < -1 || opt.def_compress_algo > 2 )
 	log_error(_("compress algorithm must be in range %d..%d\n"), 0, 2);
     if( opt.completes_needed < 1 )
@@ -1934,6 +1986,37 @@ main( int argc, char **argv )
        keygen_set_std_prefs(pers_compress_list,PREFTYPE_ZIP))
       log_error(_("invalid personal compress preferences\n"));
 
+    /* We don't support all possible commands with multifile yet */
+    if(multifile)
+      {
+	char *cmdname;
+
+	switch(cmd)
+	  {
+	  case aSign:
+	    cmdname="--sign";
+	    break;
+	  case aClearsign:
+	    cmdname="--clearsign";
+	    break;
+	  case aDetachedSign:
+	    cmdname="--detach-sign";
+	    break;
+	  case aSym:
+	    cmdname="--symmetric";
+	    break;
+	  case aStore:
+	    cmdname="--store";
+	    break;
+	  default:
+	    cmdname=NULL;
+	    break;
+	  }
+
+	if(cmdname)
+	  log_error(_("%s does not yet work with %s\n"),cmdname,"--multifile");
+      }
+
     if( log_get_errorcount(0) )
 	g10_exit(2);
 
@@ -2042,8 +2125,7 @@ main( int argc, char **argv )
 	&& !(cmd == aKMode && argc == 2 ) ) 
       {
         if (cmd != aCheckKeys && cmd != aListSigs && cmd != aListKeys
-            && cmd != aVerify && cmd != aVerifyFiles
-            && cmd != aSym)
+            && cmd != aVerify && cmd != aSym)
           {
             if (!sec_nrings || default_keyring) /* add default secret rings */
               keydb_add_resource ("secring" EXTSEP_S "gpg", 0, 1);
@@ -2112,15 +2194,17 @@ main( int argc, char **argv )
 	break;
 
       case aEncr: /* encrypt the given file */
-	if( argc > 1 )
-	    wrong_args(_("--encrypt [filename]"));
-	if( (rc = encode_crypt(fname,remusr)) )
-	    log_error("%s: encryption failed: %s\n", print_fname_stdin(fname), g10_errstr(rc) );
+	if(multifile)
+	  encode_crypt_files(argc, argv, remusr);
+	else
+	  {
+	    if( argc > 1 )
+	      wrong_args(_("--encrypt [filename]"));
+	    if( (rc = encode_crypt(fname,remusr)) )
+	      log_error("%s: encryption failed: %s\n",
+			print_fname_stdin(fname), g10_errstr(rc) );
+	  }
 	break;
-
-      case aEncrFiles: /* encrypt the given files */
-        encode_crypt_files(argc, argv, remusr);
-    	break;
           
       case aSign: /* sign the given file */
 	sl = NULL;
@@ -2173,25 +2257,29 @@ main( int argc, char **argv )
 	break;
 
       case aVerify:
-	if( (rc = verify_signatures( argc, argv ) ))
-	    log_error("verify signatures failed: %s\n", g10_errstr(rc) );
-	break;
-
-      case aVerifyFiles:
-	if( (rc = verify_files( argc, argv ) ))
-	    log_error("verify files failed: %s\n", g10_errstr(rc) );
+	if(multifile)
+	  {
+	    if( (rc = verify_files( argc, argv ) ))
+	      log_error("verify files failed: %s\n", g10_errstr(rc) );
+	  }
+	else
+	  {
+	    if( (rc = verify_signatures( argc, argv ) ))
+	      log_error("verify signatures failed: %s\n", g10_errstr(rc) );
+	  }
 	break;
 
       case aDecrypt:
-	if( argc > 1 )
-	    wrong_args(_("--decrypt [filename]"));
-	if( (rc = decrypt_message( fname ) ))
-	    log_error("decrypt_message failed: %s\n", g10_errstr(rc) );
+        if(multifile)
+	  decrypt_messages(argc, argv);
+	else
+	  {
+	    if( argc > 1 )
+	      wrong_args(_("--decrypt [filename]"));
+	    if( (rc = decrypt_message( fname ) ))
+	      log_error("decrypt_message failed: %s\n", g10_errstr(rc) );
+	  }
 	break;
-
-      case aDecryptFiles:
-        decrypt_messages(argc, argv);
-        break;
             
       case aSignKey: /* sign the key given as argument */
 	if( argc != 1 )
@@ -2321,7 +2409,6 @@ main( int argc, char **argv )
 	break;
 
       case aExport:
-      case aExportAll:
       case aSendKeys:
       case aRecvKeys:
 	sl = NULL;
@@ -2349,7 +2436,6 @@ main( int argc, char **argv )
 	sl = NULL;
 	for( ; argc; argc--, argv++ )
 	  append_to_strlist2( &sl, *argv, utf8_strings );
-
 	rc=keyserver_search( sl );
 	if(rc)
 	  log_error(_("keyserver search failed: %s\n"),g10_errstr(rc));
