@@ -1,5 +1,6 @@
 /* getkey.c -  Get a key from the database
- * Copyright (C) 1998, 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002,
+ *               2003 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -38,7 +39,7 @@
 #define MAX_UID_CACHE_ENTRIES  200
 
 #if MAX_PK_CACHE_ENTRIES < 2
-  #error We need the cache for key creation
+#error We need the cache for key creation
 #endif
 
 
@@ -83,7 +84,7 @@ typedef struct keyid_list {
 #endif
 
 #if MAX_UID_CACHE_ENTRIES < 5
-    #error we really need the userid cache
+#error we really need the userid cache
 #endif
 typedef struct user_id_db {
     struct user_id_db *next;
@@ -96,7 +97,6 @@ static int uid_cache_entries;	/* number of entries in uid cache */
 
 static void merge_selfsigs( KBNODE keyblock );
 static int lookup( GETKEY_CTX ctx, KBNODE *ret_keyblock, int secmode );
-static int get_pubkey_direct (PKT_public_key *pk, u32 *keyid);
 
 #if 0
 static void
@@ -119,7 +119,7 @@ print_stats()
 void
 cache_public_key( PKT_public_key *pk )
 {
-  #if MAX_PK_CACHE_ENTRIES
+#if MAX_PK_CACHE_ENTRIES
     pk_cache_entry_t ce;
     u32 keyid[2];
 
@@ -158,7 +158,7 @@ cache_public_key( PKT_public_key *pk )
     ce->pk = copy_public_key( NULL, pk );
     ce->keyid[0] = keyid[0];
     ce->keyid[1] = keyid[1];
-  #endif
+#endif
 }
 
 
@@ -266,7 +266,7 @@ cache_user_id( KBNODE keyblock )
 void
 getkey_disable_caches()
 {
-  #if MAX_PK_CACHE_ENTRIES
+#if MAX_PK_CACHE_ENTRIES
     {
 	pk_cache_entry_t ce, ce2;
 
@@ -279,7 +279,7 @@ getkey_disable_caches()
 	pk_cache_entries = 0;
 	pk_cache = NULL;
     }
-  #endif
+#endif
     /* fixme: disable user id cache ? */
 }
 
@@ -319,7 +319,7 @@ get_pubkey( PKT_public_key *pk, u32 *keyid )
     int internal = 0;
     int rc = 0;
 
-  #if MAX_PK_CACHE_ENTRIES
+#if MAX_PK_CACHE_ENTRIES
     {	/* Try to get it from the cache */
 	pk_cache_entry_t ce;
 	for( ce = pk_cache; ce; ce = ce->next ) {
@@ -330,7 +330,7 @@ get_pubkey( PKT_public_key *pk, u32 *keyid )
 	    }
 	}
     }
-  #endif
+#endif
     /* more init stuff */
     if( !pk ) {
 	pk = m_alloc_clear( sizeof *pk );
@@ -375,13 +375,15 @@ get_pubkey( PKT_public_key *pk, u32 *keyid )
 
 /* Get a public key and store it into the allocated pk.  This function
    differs from get_pubkey() in that it does not do a check of the key
-   to avoid recursion.  It should be used only in very certain cases.  */
-static int
-get_pubkey_direct (PKT_public_key *pk, u32 *keyid)
+   to avoid recursion.  It should be used only in very certain cases.
+   It will only retrieve primary keys. */
+int
+get_pubkey_fast (PKT_public_key *pk, u32 *keyid)
 {
   int rc = 0;
   KEYDB_HANDLE hd;
   KBNODE keyblock;
+  u32 pkid[2];
   
   assert (pk);
 #if MAX_PK_CACHE_ENTRIES
@@ -414,20 +416,25 @@ get_pubkey_direct (PKT_public_key *pk, u32 *keyid)
       log_error ("keydb_get_keyblock failed: %s\n", g10_errstr(rc));
       return G10ERR_NO_PUBKEY;
     }
-  
+
   assert ( keyblock->pkt->pkttype == PKT_PUBLIC_KEY
            ||  keyblock->pkt->pkttype == PKT_PUBLIC_SUBKEY );
-  copy_public_key (pk, keyblock->pkt->pkt.public_key );
+
+  keyid_from_pk(keyblock->pkt->pkt.public_key,pkid);
+  if(keyid[0]==pkid[0] && keyid[1]==pkid[1])
+    copy_public_key (pk, keyblock->pkt->pkt.public_key );
+  else
+    rc=G10ERR_NO_PUBKEY;
+
   release_kbnode (keyblock);
 
   /* Not caching key here since it won't have all of the fields
      properly set. */
 
-  return 0;
+  return rc;
 }
 
 
-
 KBNODE
 get_pubkeyblock( u32 *keyid )
 {
@@ -569,7 +576,7 @@ classify_user_id2( const char *name,
     memset (desc, 0, sizeof *desc);
     *force_exact = 0;
     /* skip leading spaces.  Fixme: what is with trailing spaces? */
-    for(s = name; *s && isspace(*s); s++ )
+    for(s = name; *s && spacep (s); s++ )
 	;
 
     switch (*s) {
@@ -650,7 +657,7 @@ classify_user_id2( const char *name,
             }
 
 	    /* check if a hexadecimal number is terminated by EOS or blank */
-	    if (hexlength && s[hexlength] && !isspace(s[hexlength])) {
+	    if (hexlength && s[hexlength] && !spacep(s+hexlength)) {
 		if (hexprefix)	    /* a "0x" prefix without correct */
 		    return 0;	    /* termination is an error */
 		else		    /* The first chars looked like */
@@ -734,6 +741,27 @@ classify_user_id (const char *name, KEYDB_SEARCH_DESC *desc)
     return classify_user_id2 (name, desc, &dummy);
 }
 
+static int
+skip_disabled(void *dummy,u32 *keyid)
+{
+  int rc,disabled=0;
+  PKT_public_key *pk=m_alloc_clear(sizeof(PKT_public_key));
+
+  rc = get_pubkey(pk, keyid);
+  if(rc)
+    {
+      log_error("error checking disabled status of %08lX: %s\n",
+ 		(ulong)keyid[1],g10_errstr(rc));
+      goto leave;
+    }
+ 
+  disabled=pk_is_disabled(pk);
+
+ leave:
+  free_public_key(pk);
+  return disabled;
+}
+
 /****************
  * Try to get the pubkey by the userid. This function looks for the
  * first pubkey certificate which has the given name in a user_id.
@@ -745,7 +773,8 @@ classify_user_id (const char *name, KEYDB_SEARCH_DESC *desc)
 
 static int
 key_byname( GETKEY_CTX *retctx, STRLIST namelist,
-	    PKT_public_key *pk, PKT_secret_key *sk, int secmode,
+	    PKT_public_key *pk, PKT_secret_key *sk,
+	    int secmode, int include_disabled,
             KBNODE *ret_kb, KEYDB_HANDLE *ret_kdbhd )
 {
     int rc = 0;
@@ -778,6 +807,13 @@ key_byname( GETKEY_CTX *retctx, STRLIST namelist,
 	    m_free (ctx);
 	    return G10ERR_INV_USER_ID;
 	}
+	if(!include_disabled
+ 	   && ctx->items[n].mode!=KEYDB_SEARCH_MODE_SHORT_KID
+ 	   && ctx->items[n].mode!=KEYDB_SEARCH_MODE_LONG_KID
+ 	   && ctx->items[n].mode!=KEYDB_SEARCH_MODE_FPR16
+	   && ctx->items[n].mode!=KEYDB_SEARCH_MODE_FPR20
+	   && ctx->items[n].mode!=KEYDB_SEARCH_MODE_FPR)
+ 	  ctx->items[n].skipfnc=skip_disabled;
     }
 
     ctx->kr_handle = keydb_new (secmode);
@@ -828,13 +864,14 @@ key_byname( GETKEY_CTX *retctx, STRLIST namelist,
 int
 get_pubkey_byname (PKT_public_key *pk,
 		   const char *name, KBNODE *ret_keyblock,
-                   KEYDB_HANDLE *ret_kdbhd ) 
+                   KEYDB_HANDLE *ret_kdbhd, int include_disabled ) 
 {
     int rc;
     STRLIST namelist = NULL;
 
     add_to_strlist( &namelist, name );
-    rc = key_byname( NULL, namelist, pk, NULL, 0, ret_keyblock, ret_kdbhd);
+    rc = key_byname( NULL, namelist, pk, NULL, 0,
+		     include_disabled, ret_keyblock, ret_kdbhd);
     free_strlist( namelist );
     return rc;
 }
@@ -843,7 +880,7 @@ int
 get_pubkey_bynames( GETKEY_CTX *retctx, PKT_public_key *pk,
 		    STRLIST names, KBNODE *ret_keyblock )
 {
-    return key_byname( retctx, names, pk, NULL, 0, ret_keyblock, NULL);
+    return key_byname( retctx, names, pk, NULL, 0, 1, ret_keyblock, NULL);
 }
 
 int
@@ -908,6 +945,58 @@ get_pubkey_byfprint( PKT_public_key *pk,
     return rc;
 }
 
+
+/* Get a public key and store it into the allocated pk.  This function
+   differs from get_pubkey_byfprint() in that it does not do a check
+   of the key to avoid recursion.  It should be used only in very
+   certain cases.  PK may be NULL to check just for the existance of
+   the key. */
+int
+get_pubkey_byfprint_fast (PKT_public_key *pk,
+                          const byte *fprint, size_t fprint_len)
+{
+  int rc = 0;
+  KEYDB_HANDLE hd;
+  KBNODE keyblock;
+  byte fprbuf[MAX_FINGERPRINT_LEN];
+  int i;
+  
+  for (i=0; i < MAX_FINGERPRINT_LEN && i < fprint_len; i++)
+    fprbuf[i] = fprint[i];
+  while (i < MAX_FINGERPRINT_LEN) 
+    fprbuf[i++] = 0;
+
+  hd = keydb_new (0);
+  rc = keydb_search_fpr (hd, fprbuf);
+  if (rc == -1)
+    {
+      keydb_release (hd);
+      return G10ERR_NO_PUBKEY;
+    }
+  rc = keydb_get_keyblock (hd, &keyblock);
+  keydb_release (hd);
+  if (rc) 
+    {
+      log_error ("keydb_get_keyblock failed: %s\n", g10_errstr(rc));
+      return G10ERR_NO_PUBKEY;
+    }
+  
+  assert ( keyblock->pkt->pkttype == PKT_PUBLIC_KEY
+           ||  keyblock->pkt->pkttype == PKT_PUBLIC_SUBKEY );
+  if (pk)
+    copy_public_key (pk, keyblock->pkt->pkt.public_key );
+  release_kbnode (keyblock);
+
+  /* Not caching key here since it won't have all of the fields
+     properly set. */
+
+  return 0;
+}
+
+
+
+
+
 /****************
  * Search for a key with the given fingerprint and return the
  * complete keyblock which may have more than only this key.
@@ -952,7 +1041,7 @@ get_seckey_byname2( GETKEY_CTX *retctx,
 
     if( !name && opt.def_secret_key && *opt.def_secret_key ) {
 	add_to_strlist( &namelist, opt.def_secret_key );
-	rc = key_byname( retctx, namelist, NULL, sk, 1, retblock, NULL );
+	rc = key_byname( retctx, namelist, NULL, sk, 1, 1, retblock, NULL );
     }
     else if( !name ) { /* use the first one as default key */
 	struct getkey_ctx_s ctx;
@@ -967,13 +1056,18 @@ get_seckey_byname2( GETKEY_CTX *retctx,
 	ctx.items[0].mode = KEYDB_SEARCH_MODE_FIRST;
 	rc = lookup( &ctx, &kb, 1 );
         if (!rc && sk )
-            sk_from_block ( &ctx, sk, kb );
+	  {
+	    sk_from_block ( &ctx, sk, kb );
+	    if(sk->pubkey_algo==PUBKEY_ALGO_ELGAMAL
+	       || (sk->pubkey_algo==PUBKEY_ALGO_ELGAMAL_E && sk->version<4))
+	      rc=G10ERR_UNU_SECKEY;
+	  }
         release_kbnode ( kb );
 	get_seckey_end( &ctx );
     }
     else {
 	add_to_strlist( &namelist, name );
-	rc = key_byname( retctx, namelist, NULL, sk, 1, retblock, NULL );
+	rc = key_byname( retctx, namelist, NULL, sk, 1, 1, retblock, NULL );
     }
 
     free_strlist( namelist );
@@ -995,7 +1089,7 @@ int
 get_seckey_bynames( GETKEY_CTX *retctx, PKT_secret_key *sk,
 		    STRLIST names, KBNODE *ret_keyblock )
 {
-    return key_byname( retctx, names, NULL, sk, 1, ret_keyblock, NULL );
+    return key_byname( retctx, names, NULL, sk, 1, 1, ret_keyblock, NULL );
 }
 
 
@@ -1205,7 +1299,7 @@ fixup_uidnode ( KBNODE uidnode, KBNODE signode, u32 keycreated )
     uid->is_primary = 0;
     p = parse_sig_subpkt ( sig->hashed, SIGSUBPKT_PRIMARY_UID, NULL );
     if ( p && *p )
-        uid->is_primary = 1;
+        uid->is_primary = 2;
     /* We could also query this from the unhashed area if it is not in
      * the hased area and then later try to decide which is the better
      * there should be no security problem with this.
@@ -1250,7 +1344,6 @@ fixup_uidnode ( KBNODE uidnode, KBNODE signode, u32 keycreated )
     p = parse_sig_subpkt (sig->hashed, SIGSUBPKT_FEATURES, &n);
     if (p && n && (p[0] & 0x01))
         uid->mdc_feature = 1;
-            
 }
 
 static void
@@ -1259,7 +1352,7 @@ merge_selfsigs_main( KBNODE keyblock, int *r_revoked )
     PKT_public_key *pk = NULL;
     KBNODE k;
     u32 kid[2];
-    u32 sigdate = 0, uiddate=0, uiddate2;
+    u32 sigdate, uiddate, uiddate2;
     KBNODE signode, uidnode, uidnode2;
     u32 curtime = make_timestamp ();
     unsigned int key_usage = 0;
@@ -1342,7 +1435,8 @@ merge_selfsigs_main( KBNODE keyblock, int *r_revoked )
                     else {
                         sigdate = sig->timestamp;
                         signode = k;
-			sigversion = sig->version;
+			if( sig->version > sigversion )
+			  sigversion = sig->version;
 
 		    }
 		  }
@@ -1440,15 +1534,15 @@ merge_selfsigs_main( KBNODE keyblock, int *r_revoked )
     /* second pass: look at the self-signature of all user IDs */
     signode = uidnode = NULL;
     sigdate = 0; /* helper to find the latest signature in one user ID */
-    uiddate = 0; /* and over of all user IDs */
     for(k=keyblock; k && k->pkt->pkttype != PKT_PUBLIC_SUBKEY; k = k->next ) {
 	if ( k->pkt->pkttype == PKT_USER_ID ) {
             if ( uidnode && signode ) 
+	      {
                 fixup_uidnode ( uidnode, signode, keytimestamp );
+		pk->is_valid=1;
+	      }
             uidnode = k;
             signode = NULL;
-            if ( sigdate > uiddate )
-                uiddate = sigdate;
             sigdate = 0;
       	}
         else if ( k->pkt->pkttype == PKT_SIGNATURE && uidnode ) {
@@ -1520,9 +1614,10 @@ merge_selfsigs_main( KBNODE keyblock, int *r_revoked )
 
 		    ultimate_pk=m_alloc_clear(sizeof(*ultimate_pk));
 
-		    if(get_pubkey_direct(ultimate_pk,sig->keyid)==0 &&
-		       check_key_signature(keyblock,k,NULL)==0 &&
-		       get_ownertrust(ultimate_pk)==TRUST_ULTIMATE)
+		    if(get_pubkey_fast(ultimate_pk,sig->keyid)==0
+		       && check_key_signature2(keyblock,k,ultimate_pk,
+					       NULL,NULL,NULL)==0
+		       && get_ownertrust(ultimate_pk)==TRUST_ULTIMATE)
 		      {
 			free_public_key(ultimate_pk);
 			pk->is_valid=1;
@@ -1535,9 +1630,15 @@ merge_selfsigs_main( KBNODE keyblock, int *r_revoked )
 	  }
       }
 
-    /* Record the highest selfsigversion so we know if this is a v3
-       key through and through, or a v3 key with a v4 selfsig, which
-       means we can trust the preferences (if any). */
+    /* Record the highest selfsig version so we know if this is a v3
+       key through and through, or a v3 key with a v4 selfsig
+       somewhere.  This is useful in a few places to know if the key
+       must be treated as PGP2-style or OpenPGP-style.  Note that a
+       selfsig revocation with a higher version number will also raise
+       this value.  This is okay since such a revocation must be
+       issued by the user (i.e. it cannot be issued by someone else to
+       modify the key behavior.) */
+
     pk->selfsigversion=sigversion;
 
     /* Now that we had a look at all user IDs we can now get some information
@@ -1566,6 +1667,13 @@ merge_selfsigs_main( KBNODE keyblock, int *r_revoked )
         if ( x ) /* mask it down to the actual allowed usage */
             key_usage &= x; 
     }
+
+    /* Type 20 Elgamal keys and the old v3 Elgamal keys are not
+       usable. */
+    if(pk->pubkey_algo==PUBKEY_ALGO_ELGAMAL
+       || (pk->pubkey_algo==PUBKEY_ALGO_ELGAMAL_E && pk->version<4))
+      key_usage=0;
+
     pk->pubkey_usage = key_usage;
 
     if ( !key_expire_seen ) {
@@ -1603,14 +1711,38 @@ merge_selfsigs_main( KBNODE keyblock, int *r_revoked )
         if ( k->pkt->pkttype == PKT_USER_ID &&
 	     !k->pkt->pkt.user_id->attrib_data) {
             PKT_user_id *uid = k->pkt->pkt.user_id;
-            if ( uid->is_primary && uid->created > uiddate ) {
-                uiddate = uid->created;
-                uidnode = k;
-            }
-            if ( !uid->is_primary && uid->created > uiddate2 ) {
-                uiddate2 = uid->created;
-                uidnode2 = k;
-            }
+            if (uid->is_primary)
+	      {
+		if(uid->created > uiddate)
+		  {
+		    uiddate = uid->created;
+		    uidnode = k;
+		  }
+		else if(uid->created==uiddate && uidnode)
+		  {
+		    /* The dates are equal, so we need to do a
+		       different (and arbitrary) comparison.  This
+		       should rarely, if ever, happen.  It's good to
+		       try and guarantee that two different GnuPG
+		       users with two different keyrings at least pick
+		       the same primary. */
+		    if(cmp_user_ids(uid,uidnode->pkt->pkt.user_id)>0)
+		      uidnode=k;
+		  }
+	      }
+	    else
+	      {
+		if(uid->created > uiddate2)
+		  {
+		    uiddate2 = uid->created;
+		    uidnode2 = k;
+		  }
+		else if(uid->created==uiddate2 && uidnode2)
+		  {
+		    if(cmp_user_ids(uid,uidnode2->pkt->pkt.user_id)>0)
+		      uidnode2=k;
+		  }
+	      }
         }
     }
     if ( uidnode ) {
@@ -1625,23 +1757,43 @@ merge_selfsigs_main( KBNODE keyblock, int *r_revoked )
         }
     }
     else if( uidnode2 ) {
-        /* none is flagged primary - use the latest user ID we have */
+        /* none is flagged primary - use the latest user ID we have,
+	   and disambiguate with the arbitrary packet comparison. */
         uidnode2->pkt->pkt.user_id->is_primary = 1;
     }
     else
       {
-	/* None of our uids were self-signed, so pick the first one to
-	   be the primary.  This is the best we can do here since
-	   there are no self sigs to date the uids. */
+	/* None of our uids were self-signed, so pick the one that
+	   sorts first to be the primary.  This is the best we can do
+	   here since there are no self sigs to date the uids. */
+
+	uidnode = NULL;
 
 	for(k=keyblock; k && k->pkt->pkttype != PKT_PUBLIC_SUBKEY;
 	    k = k->next )
 	  {
-	    if(k->pkt->pkttype==PKT_USER_ID &&
-	       !k->pkt->pkt.user_id->attrib_data)
+	    if(k->pkt->pkttype==PKT_USER_ID
+	       && !k->pkt->pkt.user_id->attrib_data)
 	      {
-		k->pkt->pkt.user_id->is_primary=1;
-		break;
+		if(!uidnode)
+		  {
+		    uidnode=k;
+		    uidnode->pkt->pkt.user_id->is_primary=1;
+		    continue;
+		  }
+		else
+		  {
+		    if(cmp_user_ids(k->pkt->pkt.user_id,
+				    uidnode->pkt->pkt.user_id)>0)
+		      {
+			uidnode->pkt->pkt.user_id->is_primary=0;
+			uidnode=k;
+			uidnode->pkt->pkt.user_id->is_primary=1;
+		      }
+		    else
+		      k->pkt->pkt.user_id->is_primary=0; /* just to be
+							    safe */
+		  }
 	      }
 	  }
       }
@@ -1736,6 +1888,14 @@ merge_selfsigs_subkey( KBNODE keyblock, KBNODE subnode )
         if ( x ) /* mask it down to the actual allowed usage */
             key_usage &= x; 
     }
+
+    /* Type 20 Elgamal subkeys, any subkey on a type 20 primary, or
+       any subkey on an old v3 Elgamal(e) primary are not usable. */
+    if(mainpk->pubkey_algo==PUBKEY_ALGO_ELGAMAL
+       || subpk->pubkey_algo==PUBKEY_ALGO_ELGAMAL
+       || (mainpk->pubkey_algo==PUBKEY_ALGO_ELGAMAL_E && mainpk->version<4))
+      key_usage=0;
+
     subpk->pubkey_usage = key_usage;
     
     p = parse_sig_subpkt (sig->hashed, SIGSUBPKT_KEY_EXPIRE, NULL);
@@ -1783,21 +1943,6 @@ merge_selfsigs( KBNODE keyblock )
     }
 
     merge_selfsigs_main ( keyblock, &revoked );
-    main_pk = keyblock->pkt->pkt.public_key;
-    if ( revoked ) {
-        /* if the primary key has been revoked we better set the revoke
-         * flag on that key and all subkeys */
-        for(k=keyblock; k; k = k->next ) {
-            if ( k->pkt->pkttype == PKT_PUBLIC_KEY
-                || k->pkt->pkttype == PKT_PUBLIC_SUBKEY ) {
-                PKT_public_key *pk = k->pkt->pkt.public_key;
-                pk->is_revoked = 1;
-                pk->main_keyid[0] = main_pk->main_keyid[0];
-                pk->main_keyid[1] = main_pk->main_keyid[1];
-            }
-	}
-        return;
-    }
 
     /* now merge in the data from each of the subkeys */
     for(k=keyblock; k; k = k->next ) {
@@ -1806,12 +1951,25 @@ merge_selfsigs( KBNODE keyblock )
         }
     }
 
-    /* If the main key is not valid, then the subkeys aren't either,
-       even if they have binding sigs. */
-    if(!main_pk->is_valid)
-      for(k=keyblock; k; k=k->next)
-	if(k->pkt->pkttype==PKT_PUBLIC_SUBKEY)
-	  k->pkt->pkt.public_key->is_valid=0;
+    main_pk = keyblock->pkt->pkt.public_key;
+    if ( revoked || main_pk->has_expired || !main_pk->is_valid ) {
+        /* if the primary key is revoked, expired, or invalid we
+         * better set the appropriate flags on that key and all
+         * subkeys */
+        for(k=keyblock; k; k = k->next ) {
+            if ( k->pkt->pkttype == PKT_PUBLIC_KEY
+                || k->pkt->pkttype == PKT_PUBLIC_SUBKEY ) {
+                PKT_public_key *pk = k->pkt->pkt.public_key;
+		if(!main_pk->is_valid)
+		  pk->is_valid = 0;
+		if(revoked)
+		  pk->is_revoked = 1;
+                if(main_pk->has_expired)
+		  pk->has_expired = main_pk->has_expired;
+            }
+	}
+	return;
+    }
 
     /* set the preference list of all keys to those of the primary real
      * user ID.  Note: we use these preferences when we don't know by
@@ -1937,8 +2095,9 @@ premerge_public_with_secret ( KBNODE pubblock, KBNODE secblock )
             if ( !sec ) {
                 KBNODE next, ll;
 
-                log_info ( "no secret subkey "
-                           "for public subkey %08lX - ignoring\n",  
+                if (opt.verbose)
+                  log_info ( _("no secret subkey "
+                               "for public subkey %08lX - ignoring\n"),  
                            (ulong)keyid_from_pk (pk,NULL) );
                 /* we have to remove the subkey in this case */
                 assert ( last );
@@ -2002,13 +2161,14 @@ finish_lookup (GETKEY_CTX ctx)
     KBNODE k;
     KBNODE foundk = NULL;
     PKT_user_id *foundu = NULL;
-  #define USAGE_MASK  (PUBKEY_USAGE_SIG|PUBKEY_USAGE_ENC)
+#define USAGE_MASK  (PUBKEY_USAGE_SIG|PUBKEY_USAGE_ENC)
     unsigned int req_usage = ( ctx->req_usage & USAGE_MASK );
     /* Request the primary if we're certifying another key, and also
        if signing data while --pgp6 or --pgp7 is on since pgp 6 and 7
-       do not understand signatures made by a signing subkey. */
+       do not understand signatures made by a signing subkey.  PGP 8
+       does. */
     int req_prim = (ctx->req_usage & PUBKEY_USAGE_CERT) ||
-      ((opt.pgp6 || opt.pgp7) && (ctx->req_usage & PUBKEY_USAGE_SIG));
+      ((PGP6 || PGP7) && (ctx->req_usage & PUBKEY_USAGE_SIG));
     u32 latest_date;
     KBNODE latest_key;
     u32 curtime = make_timestamp ();
