@@ -180,7 +180,7 @@ hkp_export( STRLIST users )
 		   opt.keyserver_host, g10_errstr(rc) );
     }
     else {
-      #if 1
+#if 1
 	if( opt.verbose ) {
 	    int c;
 	    while( (c=iobuf_get(hd.fp_read)) != EOF )
@@ -189,7 +189,7 @@ hkp_export( STRLIST users )
               else
                 putchar ( '?' );
 	}
-      #endif
+#endif
 	if( (status/100) == 2 )
 	    log_info(_("success sending to `%s' (status=%u)\n"),
 					opt.keyserver_host, status  );
@@ -366,7 +366,7 @@ parse_hkp_index(IOBUF buffer,char *line)
     {
       m_free(key);
       m_free(uid);
-      log_error(_("this keyserver is not fully HKP compatible\n"));
+      log_error(_("this keyserver does not support --search-keys\n"));
       return -1;
     }
 
@@ -598,7 +598,10 @@ int hkp_search(STRLIST tokens)
 
 	  ret=parse_hkp_index(buffer,line);
 	  if(ret==-1)
-	    break;
+	    {
+	      rc=G10ERR_KEYSERVER;
+	      break;
+	    }
 
 	  if(rc!=0)
 	    count+=ret;
