@@ -1,5 +1,5 @@
 /* keylist.c
- * Copyright (C) 1998, 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
+ * Copyright (C) 1998,1999,2000,2001,2002,2003 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -57,10 +57,31 @@ static FILE *attrib_fp=NULL;
 void
 public_key_list( STRLIST list )
 {
-    if( !list )
-	list_all(0);
-    else
-	list_one( list, 0 );
+  if(opt.with_colons)
+    {
+      byte trust_model;
+      ulong created,nextcheck;
+
+      read_trust_options(&trust_model,&created,&nextcheck);
+
+      printf("tru:");
+
+      if(nextcheck && nextcheck <= make_timestamp())
+	printf("o");
+      if(trust_model!=0)
+	printf("t");
+
+      /* We don't show marginals, completes, or cert_depth since
+	 they are not accurate in 1.2.x - they are not updated when
+	 the trustdb is rebuilt with different options. */
+
+      printf(":%d:%lu:%lu\n",trust_model,created,nextcheck);
+    }
+
+  if( !list )
+    list_all(0);
+  else
+    list_one( list, 0 );
 }
 
 void
@@ -86,12 +107,11 @@ show_policy_url(PKT_signature *sig,int indent)
       for(i=0;i<indent;i++)
 	putchar(' ');
 
-      /* This isn't UTF8 as it is a URL(?) */
       if(crit)
 	printf(_("Critical signature policy: "));
       else
 	printf(_("Signature policy: "));
-      print_string(stdout,p,len,0);
+      print_utf8_string(stdout,p,len);
       printf("\n");
     }
 }
@@ -187,16 +207,20 @@ list_all( int secret )
             log_error ("keydb_get_keyblock failed: %s\n", g10_errstr(rc));
             goto leave;
         }
-        resname = keydb_get_resource_name (hd);
-	if (lastresname != resname ) {
-	    int i;
+	if(!opt.with_colons)
+	  {
+	    resname = keydb_get_resource_name (hd);
+	    if (lastresname != resname )
+	      {
+		int i;
 
-	    printf("%s\n", resname );
-	    for(i=strlen(resname); i; i-- )
-		putchar('-');
-	    putchar('\n');
-            lastresname = resname;
-	}
+		printf("%s\n", resname );
+		for(i=strlen(resname); i; i-- )
+		  putchar('-');
+		putchar('\n');
+		lastresname = resname;
+	      }
+	  }
         merge_keys_and_selfsig( keyblock );
 	list_keyblock( keyblock, secret, opt.fingerprint,
 		       opt.check_sigs?&stats:NULL);
@@ -222,7 +246,7 @@ list_one( STRLIST names, int secret )
     KBNODE keyblock = NULL;
     GETKEY_CTX ctx;
     const char *resname;
-    char *keyring_str = N_("Keyring");
+    char *keyring_str = _("Keyring");
     int i;
     struct sig_stats stats;
 
@@ -245,7 +269,7 @@ list_one( STRLIST names, int secret )
 	    return;
 	}
 	do {
-	    if (opt.show_keyring) {
+	    if (opt.show_keyring && !opt.with_colons) {
 		resname = keydb_get_resource_name (get_ctx_handle(ctx));
 		printf("%s: %s\n", keyring_str, resname);
 		for(i = strlen(resname) + strlen(keyring_str) + 2; i; i-- )
@@ -265,7 +289,7 @@ list_one( STRLIST names, int secret )
 	    return;
 	}
 	do {
-	    if (opt.show_keyring) {
+	  if (opt.show_keyring && !opt.with_colons) {
 		resname = keydb_get_resource_name (get_ctx_handle(ctx));
 		printf("%s: %s\n", keyring_str, resname);
 		for(i = strlen(resname) + strlen(keyring_str) + 2; i; i-- )
@@ -315,14 +339,18 @@ print_capabilities (PKT_public_key *pk, PKT_secret_key *sk, KBNODE keyblock)
 	}
     }
 
-    if ( keyblock ) { /* figure our the usable capabilities */
+    if ( keyblock ) { /* figure out the usable capabilities */
         KBNODE k;
-        int enc=0, sign=0, cert=0;
+        int enc=0, sign=0, cert=0, disabled=0;
 
         for (k=keyblock; k; k = k->next ) {
             if ( k->pkt->pkttype == PKT_PUBLIC_KEY 
                  || k->pkt->pkttype == PKT_PUBLIC_SUBKEY ) {
                 pk = k->pkt->pkt.public_key;
+
+		if(pk->is_primary)
+		  disabled=pk_is_disabled(pk);
+
                 if ( pk->is_valid && !pk->is_revoked && !pk->has_expired ) {
                     if ( pk->pubkey_usage & PUBKEY_USAGE_ENC )
                         enc = 1;
@@ -356,17 +384,20 @@ print_capabilities (PKT_public_key *pk, PKT_secret_key *sk, KBNODE keyblock)
             putchar ('S');
         if (cert)
             putchar ('C');
+        if (disabled)
+            putchar ('D');
     }
+
     putchar(':');
 }
 
-static void dump_attribs(const PKT_user_id *uid,
-			 PKT_public_key *pk,PKT_secret_key *sk)
+void
+dump_attribs(const PKT_user_id *uid,PKT_public_key *pk,PKT_secret_key *sk)
 {
   int i;
 
   if(!attrib_fp)
-    BUG();
+    return;
 
   for(i=0;i<uid->numattribs;i++)
     {
@@ -442,8 +473,6 @@ list_keyblock_print ( KBNODE keyblock, int secret, int fpr, void *opaque )
 
     for( kbctx=NULL; (node=walk_kbnode( keyblock, &kbctx, 0)) ; ) {
 	if( node->pkt->pkttype == PKT_USER_ID && !opt.fast_list_mode ) {
-	    if(attrib_fp && node->pkt->pkt.user_id->attrib_data!=NULL)
-	      dump_attribs(node->pkt->pkt.user_id,pk,sk);
             /* don't list revoked or expired UIDS unless we are in
              * verbose mode and signature listing has not been
              * requested */
@@ -452,6 +481,9 @@ list_keyblock_print ( KBNODE keyblock, int secret, int fpr, void *opaque )
 		  node->pkt->pkt.user_id->is_expired ))
                 continue; 
 
+	    if(attrib_fp && node->pkt->pkt.user_id->attrib_data!=NULL)
+	      dump_attribs(node->pkt->pkt.user_id,pk,sk);
+
 	    if( any ) 
                 printf("uid%*s", 28, "");
 
@@ -637,7 +669,7 @@ list_keyblock_colon( KBNODE keyblock, int secret, int fpr )
 	pk = NULL;
 	sk = node->pkt->pkt.secret_key;
 	keyid_from_sk( sk, keyid );
-        printf("sec:u:%u:%d:%08lX%08lX:%s:%s:::",
+        printf("sec::%u:%d:%08lX%08lX:%s:%s:::",
 		    nbits_from_sk( sk ),
 		    sk->pubkey_algo,
 		    (ulong)keyid[0],(ulong)keyid[1],
@@ -650,7 +682,6 @@ list_keyblock_colon( KBNODE keyblock, int secret, int fpr )
 	sk = NULL;
 	keyid_from_pk( pk, keyid );
         fputs( "pub:", stdout );
-        trustletter = 0;
         if ( !pk->is_valid )
             putchar ('i');
         else if ( pk->is_revoked )
@@ -702,30 +733,26 @@ list_keyblock_colon( KBNODE keyblock, int secret, int fpr )
              */
 	    if( any ) {
 	        char *str=node->pkt->pkt.user_id->attrib_data?"uat":"uid";
-                if ( node->pkt->pkt.user_id->is_revoked )
+		/* If we're listing a secret key, leave out the
+		   validity values for now.  This is handled better in
+		   1.9. */
+		if ( sk )
+		    printf("%s:::::::::",str);
+		else if ( node->pkt->pkt.user_id->is_revoked )
         	    printf("%s:r::::::::",str);
                 else if ( node->pkt->pkt.user_id->is_expired )
         	    printf("%s:e::::::::",str);
-		else if ( opt.no_expensive_trust_checks ) {
+		else if ( opt.no_expensive_trust_checks )
         	    printf("%s:::::::::",str);
-	        }
                 else {
-		    byte namehash[20];
+		    int uid_validity;
 
-		    if( pk && !ulti_hack ) {
-			if( node->pkt->pkt.user_id->attrib_data )
-			    rmd160_hash_buffer( namehash,
-					   node->pkt->pkt.user_id->attrib_data,
-					   node->pkt->pkt.user_id->attrib_len);
-			else
-			    rmd160_hash_buffer( namehash,
-					    node->pkt->pkt.user_id->name,
-					    node->pkt->pkt.user_id->len  );
-			trustletter = get_validity_info( pk, namehash );
-		    }
+		    if( pk && !ulti_hack )
+		      uid_validity=get_validity_info (pk,
+						      node->pkt->pkt.user_id);
 		    else
-			trustletter = 'u';
-		    printf("%s:%c::::::::",str,trustletter);
+			uid_validity = 'u';
+		    printf("%s:%c::::::::",str,uid_validity);
                 }
 	    }
 	    if(node->pkt->pkt.user_id->attrib_data)
@@ -774,7 +801,9 @@ list_keyblock_colon( KBNODE keyblock, int secret, int fpr )
             else if ( opt.fast_list_mode || opt.no_expensive_trust_checks )
                 ;
             else {
-                printf("%c", trustletter );
+	        /* trustletter should always be defined here */
+	        if(trustletter)
+		  printf("%c", trustletter );
             }
             printf(":%u:%d:%08lX%08lX:%s:%s:",
 			nbits_from_pk( pk2 ),
@@ -909,7 +938,7 @@ list_keyblock_colon( KBNODE keyblock, int secret, int fpr )
  * Reorder the keyblock so that the primary user ID (and not attribute
  * packet) comes first.  Fixme: Replace this by a generic sort
  * function.  */
-static void
+void
 reorder_keyblock (KBNODE keyblock)
 {
     KBNODE primary = NULL, primary0 = NULL, primary2 = NULL;
