@@ -861,6 +861,7 @@ keyring_search (KEYRING_HANDLE hd, KEYDB_SEARCH_DESC *desc, size_t ndesc)
   PKT_user_id *uid = NULL;
   PKT_public_key *pk = NULL;
   PKT_secret_key *sk = NULL;
+  u32 aki[2];
 
   /* figure out what information we need */
   need_uid = need_words = need_keyid = need_fpr = any_skip = 0;
@@ -963,7 +964,6 @@ keyring_search (KEYRING_HANDLE hd, KEYDB_SEARCH_DESC *desc, size_t ndesc)
     {
       byte afp[MAX_FINGERPRINT_LEN];
       size_t an;
-      u32 aki[2];
 
       if (pkt.pkttype == PKT_PUBLIC_KEY  || pkt.pkttype == PKT_SECRET_KEY) 
         {
@@ -1316,6 +1316,10 @@ keyring_rebuild_cache (void *token)
   memset (&desc, 0, sizeof desc);
   desc.mode = KEYDB_SEARCH_MODE_FIRST;
 
+  rc=keyring_lock (hd, 1);
+  if(rc)
+    goto leave;
+
   while ( !(rc = keyring_search (hd, &desc, 1)) )
     {
       desc.mode = KEYDB_SEARCH_MODE_NEXT;
@@ -1363,6 +1367,13 @@ keyring_rebuild_cache (void *token)
         {
           if (node->pkt->pkttype == PKT_SIGNATURE)
             {
+	      /* Note that this doesn't cache the result of a
+		 revocation issued by a designated revoker.  This is
+		 because the pk in question does not carry the revkeys
+		 as we haven't merged the key and selfsigs.  It is
+		 questionable whether this matters very much since
+		 there are very very few designated revoker revocation
+		 packets out there. */
               check_key_signature (keyblock, node, NULL);
               sigcount++;
             }
@@ -1410,6 +1421,7 @@ keyring_rebuild_cache (void *token)
   m_free (tmpfilename);  
   m_free (bakfilename);  
   release_kbnode (keyblock);
+  keyring_lock (hd, 0);
   keyring_release (hd);
   return rc;
 }
