@@ -1,5 +1,5 @@
 /* keyserver.c - generic keyserver code
- * Copyright (C) 2001, 2002 Free Software Foundation, Inc.
+ * Copyright (C) 2001, 2002, 2003 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -156,7 +156,8 @@ parse_keyserver_uri(char *uri,const char *configname,unsigned int configlineno)
       opt.keyserver_scheme="hkp";
       opt.keyserver_options.broken_http_proxy=1;
     }
-  else if(ascii_strcasecmp(opt.keyserver_scheme,"x-hkp")==0)
+  else if(ascii_strcasecmp(opt.keyserver_scheme,"x-hkp")==0
+	  || ascii_strcasecmp(opt.keyserver_scheme,"http")==0)
     {
       /* Canonicalize this to "hkp" so it works with both the internal
 	 and external keyserver interface. */
@@ -189,11 +190,16 @@ parse_keyserver_uri(char *uri,const char *configname,unsigned int configlineno)
 	  ch=opt.keyserver_port;
 	  while(*ch!='\0')
 	    {
-	      if(!isdigit(*ch))
+	      if(!digitp(ch))
 		return G10ERR_BAD_URI;
 
 	      ch++;
 	    }
+
+	  /* It would seem to be reasonable to limit the range of the
+	     ports to values between 1-65535, but RFC 1738 and 1808
+	     imply there is no limit.  Of course, the real world has
+	     limits. */
 	}
 
       /* (any path part of the URI is discarded for now as no keyserver
@@ -564,7 +570,7 @@ keyserver_spawn(int action,STRLIST list,
 
       /* remove trailing whitespace */
       plen=strlen(ptr);
-      while(plen>0 && isspace(ptr[plen-1]))
+      while(plen>0 && ascii_isspace(ptr[plen-1]))
 	plen--;
       plen[ptr]='\0';
 
@@ -679,6 +685,10 @@ keyserver_work(int action,STRLIST list,KEYDB_SEARCH_DESC *desc,int count)
 	  log_error(_("no keyserver known (use option --keyserver)\n"));
 	  return G10ERR_BAD_URI;
 	}
+      else if(opt.keyserver_port && (strlen(opt.keyserver_port)>5
+				     || atoi(opt.keyserver_port)<1
+				     || atoi(opt.keyserver_port)>65535))
+	return G10ERR_BAD_URI;
       else
 	{
 	  void *stats_handle = import_new_stats_handle ();
@@ -728,6 +738,11 @@ keyserver_work(int action,STRLIST list,KEYDB_SEARCH_DESC *desc,int count)
 		    action==SEARCH?"search":"unknown",
 		    opt.keyserver_scheme);
 
+	case KEYSERVER_VERSION_ERROR:
+	  log_error(_("gpgkeys_%s does not support handler version %d\n"),
+		    opt.keyserver_scheme,KEYSERVER_PROTO_VERSION);
+	  break;
+
 	case KEYSERVER_INTERNAL_ERROR:
 	default:
 	  log_error(_("keyserver internal error\n"));
@@ -976,17 +991,23 @@ keyserver_refresh(STRLIST users)
   if(rc)
     return rc;
 
-  if(count==1)
-    log_info(_("refreshing 1 key from %s\n"),opt.keyserver_uri);
-  else
-    log_info(_("refreshing %d keys from %s\n"),count,opt.keyserver_uri);
-
   if(count>0)
-    rc=keyserver_work(GET,NULL,desc,count);
+    {
+      if(opt.keyserver_uri)
+	{
+	  if(count==1)
+	    log_info(_("refreshing 1 key from %s\n"),opt.keyserver_uri);
+	  else
+	    log_info(_("refreshing %d keys from %s\n"),
+		     count,opt.keyserver_uri);
+	}
+
+      rc=keyserver_work(GET,NULL,desc,count);
+    }
 
   m_free(desc);
 
-  return 0;
+  return rc;
 }
 
 int
@@ -1067,6 +1088,9 @@ keyserver_search_prompt(IOBUF buffer,int count,const char *searchstr)
 
       if(i%10==0 || rl==0)
 	{
+          /* A flush doesn't change anything but might be helpful for 
+             some applications. */
+          fflush (stdout);
 	  answer=cpr_get_no_help("keysearch.prompt",
 				 _("Enter number(s), N)ext, or Q)uit > "));
 	  /* control-d */
