@@ -25,7 +25,7 @@
 #include <errno.h>
 #include <assert.h>
 #ifdef HAVE_DOSISH_SYSTEM
-  #include <fcntl.h> /* for setmode() */
+#include <fcntl.h> /* for setmode() */
 #endif
 
 #include "util.h"
@@ -85,9 +85,9 @@ handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
     else if( !*fname || (*fname=='-' && !fname[1])) {
 	/* no filename or "-" given; write to stdout */
 	fp = stdout;
-      #ifdef HAVE_DOSISH_SYSTEM
+#ifdef HAVE_DOSISH_SYSTEM
 	setmode ( fileno(fp) , O_BINARY );
-      #endif
+#endif
     }
     else {
 	while( !overwrite_filep (fname) ) {
@@ -127,10 +127,10 @@ handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
 		}
 		if( mfx->md )
 		    md_putc(mfx->md, c );
-	      #ifndef HAVE_DOSISH_SYSTEM
+#ifndef HAVE_DOSISH_SYSTEM
 		if( c == '\r' )  /* convert to native line ending */
 		    continue;	 /* fixme: this hack might be too simple */
-	      #endif
+#endif
 		if( fp ) {
 		    if( putc( c, fp ) == EOF ) {
 			log_error("Error writing to `%s': %s\n",
@@ -174,10 +174,10 @@ handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
 	    while( (c = iobuf_get(pt->buf)) != -1 ) {
 		if( mfx->md )
 		    md_putc(mfx->md, c );
-	      #ifndef HAVE_DOSISH_SYSTEM
+#ifndef HAVE_DOSISH_SYSTEM
 		if( convert && c == '\r' )
 		    continue; /* fixme: this hack might be too simple */
-	      #endif
+#endif
 		if( fp ) {
 		    if( putc( c, fp ) == EOF ) {
 			log_error("Error writing to `%s': %s\n",
@@ -327,11 +327,13 @@ int
 ask_for_detached_datafile( MD_HANDLE md, MD_HANDLE md2,
 			   const char *inname, int textmode )
 {
+    progress_filter_context_t pfx;
     char *answer = NULL;
     IOBUF fp;
     int rc = 0;
 
-    fp = open_sigfile( inname ); /* open default file */
+    fp = open_sigfile( inname, &pfx ); /* open default file */
+
     if( !fp && !opt.batch ) {
 	int any=0;
 	tty_printf(_("Detached signature.\n"));
@@ -366,7 +368,6 @@ ask_for_detached_datafile( MD_HANDLE md, MD_HANDLE md2,
     do_hash( md, md2, fp, textmode );
     iobuf_close(fp);
 
-
   leave:
     m_free(answer);
     return rc;
@@ -382,12 +383,13 @@ int
 hash_datafiles( MD_HANDLE md, MD_HANDLE md2, STRLIST files,
 		const char *sigfilename, int textmode )
 {
+    progress_filter_context_t pfx;
     IOBUF fp;
     STRLIST sl;
 
     if( !files ) {
 	/* check whether we can open the signed material */
-	fp = open_sigfile( sigfilename );
+	fp = open_sigfile( sigfilename, &pfx );
 	if( fp ) {
 	    do_hash( md, md2, fp, textmode );
 	    iobuf_close(fp);
@@ -405,14 +407,10 @@ hash_datafiles( MD_HANDLE md, MD_HANDLE md2, STRLIST files,
 						print_fname_stdin(sl->d));
 	    return G10ERR_OPEN_FILE;
 	}
+        handle_progress (&pfx, fp, sl->d);
 	do_hash( md, md2, fp, textmode );
 	iobuf_close(fp);
     }
 
     return 0;
 }
-
-
-
-
-
