@@ -266,6 +266,12 @@ do_sign( PKT_secret_key *sk, PKT_signature *sig,
 	    return G10ERR_TIME_CONFLICT;
     }
 
+    /* For safety, only allow revocation sigs from Elgamal
+       sign+encrypt keys.  Note that this allows for Elgamal
+       designated revocations as well, but that's arguably a good
+       thing. */
+    if(is_ELGAMAL(sk->pubkey_algo) && sig->sig_class!=0x20)
+      return G10ERR_UNU_SECKEY;
 
     print_pubkey_algo_note(sk->pubkey_algo);
 
@@ -773,10 +779,7 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
 
 	/* algo 0 means no compression */
 	if( compr_algo )
-	  {
-	    zfx.algo = compr_algo;
-	    iobuf_push_filter( out, compress_filter, &zfx );
-	  }
+	  push_compress_filter(out,&zfx,compr_algo);
       }
 
     /* Write the one-pass signature packets if needed */
@@ -1096,19 +1099,7 @@ sign_symencrypt_file (const char *fname, STRLIST locusr)
 
     /* Push the Zip filter */
     if (opt.compress)
-      {
-	int compr_algo=opt.def_compress_algo;
-
-	/* Default */
-        if(compr_algo==-1)
-	  compr_algo=DEFAULT_COMPRESS_ALGO;
-
-	if (compr_algo)
-	  {
-	    zfx.algo = compr_algo;
-	    iobuf_push_filter( out, compress_filter, &zfx );
-	  }
-      }
+      push_compress_filter(out,&zfx,opt.def_compress_algo);
 
     /* Write the one-pass signature packets */
     /*(current filters: zip - encrypt - armor)*/
